@@ -10,6 +10,7 @@ import socket
 import errno
 import platform
 from collections import deque
+from binascii import hexlify
 
 try:
     import ssl
@@ -21,6 +22,7 @@ from ...aid.sixing import *
 from ...aid.odicting import odict
 from ...aid.timing import StoreTimer
 from ...aid.consoling import getConsole
+from .. import aioing
 from ...base import storing
 
 console = getConsole()
@@ -43,7 +45,9 @@ class Client(object):
                  wlog=None,
                  store=None,
                  timeout=None,
-                 reconnectable=None, ):
+                 reconnectable=None,
+                 txes=None,
+                 rxbs=None):
         """
         Initialization method for instance.
         name = user friendly name for connection
@@ -56,6 +60,8 @@ class Client(object):
         store = store reference
         timeout = auto reconnect timeout
         reconnectable = Boolean auto reconnect if timed out
+        txes = deque of data to send
+        rxbs = bytearray of data received
         """
         self.name = name
         self.uid = uid
@@ -63,7 +69,7 @@ class Client(object):
         self.ha = ha or (host, port)
         host, port = self.ha
         self.hostname = host  # host domain name
-        host = socket.gethostbyname(host)  # ip host address
+        host = aioing.normalizeHost(host)  # ip host address
         self.ha = (host, port)
         self.bs = bufsize
         self.wlog = wlog
@@ -72,8 +78,8 @@ class Client(object):
         self.ca = (None, None)  # host address of local connection
         self._accepted = False  # attribute to support accepted property
         self.cutoff = False  # True when detect connection closed on far side
-        self.txes = deque()  # deque of data to send
-        self.rxbs = bytearray()  # byte array of data recieved
+        self.txes = txes if txes is not None else deque()  # deque of data to send
+        self.rxbs = rxbs if rxbs is not None else bytearray()  # byte array of data recieved
         self.store = store or storing.Store(stamp=0.0)
         self.timeout = timeout if timeout is not None else self.Timeout
         self.timer = StoreTimer(self.store, duration=self.timeout)
@@ -143,12 +149,12 @@ class Client(object):
         Reinit self.ha and self.hostname from ha = (host, port) or hostname port
         self.ha is of form (host, port) where host is either dns name or ip
         self.hostname is hostname as dns name
-        host eventually is host ip address output from gethostbyname()
+        host eventually is host ip address output from normalizeHost()
         """
         self.ha = ha or (hostname, port)
         hostname, port = self.ha
         self.hostname = hostname  # host domain name
-        host = socket.gethostbyname(hostname)  # ip host address
+        host = aioing.normalizeHost(hostname)  # ip host address
         self.ha = (host, port)
 
     def actualBufSizes(self):
@@ -247,19 +253,30 @@ class Client(object):
 
     close = shutclose  # alias
 
+    def refresh(self):
+        """
+        Restart timer
+        """
+        self.timer.restart()
+
     def accept(self):
         """
         Attempt nonblocking acceptance connect to .ha
         Returns True if successful
         Returns False if not so try again later
         """
+        if not self.cs:
+            self.reopen()
+            
         try:
             result = self.cs.connect_ex(self.ha)  # async connect
         except socket.error as ex:
             console.terse("socket.error = {0}\n".format(ex))
             raise
 
-        if result not in [0, errno.EISCONN]:  # not yet connected
+        if result not in [0, errno.EISCONN]:  # not connected
+            if result in (errno.EINVAL, errno.ECONNREFUSED):  # server not listening so must reopen
+                self.reopen()
             return False  # try again later
 
         # now self.cs has new virtual port see self.cs.getsockname()
@@ -301,14 +318,29 @@ class Client(object):
         Perform non blocking receive from connected socket .cs
 
         If no data then returns None
-        If connection closed then returns ''
+        If connection closed then returns empty
         Otherwise returns data
+        data is string in python2 and bytes in python3
         """
         try:
             data = self.cs.recv(self.bs)
         except socket.error as ex:
-            if ex.errno in (errno.EAGAIN, errno.EWOULDBLOCK):
-                return None
+            # ex.args[0] is always ex.errno for better compat
+            if  ex.args[0] in (errno.EAGAIN, errno.EWOULDBLOCK):
+                return None  # blocked waiting for data
+            elif ex.args[0] in (errno.ECONNRESET,
+                                errno.ENETRESET,
+                                errno.ENETUNREACH,
+                                errno.EHOSTUNREACH,
+                                errno.ENETDOWN,
+                                errno.EHOSTDOWN,
+                                errno.ETIMEDOUT,
+                                errno.ECONNREFUSED):
+                emsg = ("socket.error = {0}: Outgoer at {1} while receiving "
+                        "from {2}\n".format(ex, self.ca, self.ha))
+                console.profuse(emsg)
+                self.cutoff = True  # this signals need to close/reopen connection
+                return bytes()  # data empty
             else:
                 emsg = ("socket.error = {0}: Outgoer at {1} while receiving "
                         "from {2}\n".format(ex, self.ca, self.ha))
@@ -317,13 +349,17 @@ class Client(object):
 
         if data:  # connection open
             if console._verbosity >= console.Wordage.profuse:  # faster to check
-                cmsg = ("\nOutGoer at {0}, received from {1}:\n------------\n"
-                           "{2}\n".format(self.ca, self.ha, data.decode("UTF-8")))
+                try:
+                    load = data.decode("UTF-8")
+                except UnicodeDecodeError as ex:
+                    load = "0x{0}".format(hexlify(data).decode("ASCII"))
+                cmsg = ("Outgoer at {0}, received from {1}:\n------------\n"
+                        "{2}\n\n".format(self.ca, self.ha, load))
                 console.profuse(cmsg)
 
             if self.wlog:  # log over the wire rx
                 self.wlog.writeRx(self.ha, data)
-        else:  # data empty so connection closed on other end
+        else:  # data empty so connection closed on other end, whereas see above for blocked
             self.cutoff = True
 
         return data
@@ -369,19 +405,32 @@ class Client(object):
         """
         return (bytes(self.rxbs[index:]), len(self.rxbs))
 
-    def send(self, data, da=None):
+    def send(self, data):
         """
         Perform non blocking send on connected socket .cs.
         Return number of bytes sent
-
         data is string in python2 and bytes in python3
-        da is optional not used but for interface compat
         """
         try:
             result = self.cs.send(data) #result is number of bytes sent
         except socket.error as ex:
-            result = 0
-            if ex.errno not in (errno.EAGAIN, errno.EWOULDBLOCK):
+            # ex.args[0] is always ex.errno for better compat
+            if ex.args[0] in (errno.EAGAIN, errno.EWOULDBLOCK):
+                result = 0  # blocked try again
+            elif ex.args[0] in (errno.ECONNRESET,
+                                errno.ENETRESET,
+                                errno.ENETUNREACH,
+                                errno.EHOSTUNREACH,
+                                errno.ENETDOWN,
+                                errno.EHOSTDOWN,
+                                errno.ETIMEDOUT,
+                                errno.ECONNREFUSED):
+                emsg = ("socket.error = {0}: Outgoer at {1} while sending "
+                        "to {2} \n".format(ex, self.ca, self.ha))
+                console.profuse(emsg)
+                self.cutoff = True  # this signals need to close/reopen connection
+                result = 0
+            else:
                 emsg = ("socket.error = {0}: Outgoer at {1} while sending "
                         "to {2} \n".format(ex, self.ca, self.ha))
                 console.profuse(emsg)
@@ -389,8 +438,12 @@ class Client(object):
 
         if result:
             if console._verbosity >= console.Wordage.profuse:
-                cmsg = ("\nOutgoer at {0}, sent {1} bytes to {2}:\n------------\n"
-                        "{3}\n".format(self.ca, result, self.ha, data[:result].decode('UTF-8')))
+                try:
+                    load = data[:result].decode("UTF-8")
+                except UnicodeDecodeError as ex:
+                    load = "0x{0}".format(hexlify(data[:result]).decode("ASCII"))
+                cmsg = ("Outgoer at {0}, sent {1} bytes to {2}:\n------------\n"
+                        "{3}\n\n".format(self.ca, result, self.ha, load))
                 console.profuse(cmsg)
 
             if self.wlog:
@@ -595,22 +648,42 @@ class ClientTls(Client):
         If no data then returns None
         If connection closed then returns ''
         Otherwise returns data
+        data is string in python2 and bytes in python3
         """
         try:
             data = self.cs.recv(self.bs)
-        except ssl.SSLError as ex:
-            if ex.errno in (ssl.SSL_ERROR_WANT_READ, ssl.SSL_ERROR_WANT_WRITE):
+        except socket.error as ex:  # ssl.SSLError is a subtype of socket.error
+            # ex.args[0] is always ex.errno for better compat
+            if ex.args[0] in (ssl.SSL_ERROR_WANT_READ, ssl.SSL_ERROR_WANT_WRITE):
                 return None
+            elif ex.args[0] in (errno.ECONNRESET,
+                                errno.ENETRESET,
+                                errno.ENETUNREACH,
+                                errno.EHOSTUNREACH,
+                                errno.ENETDOWN,
+                                errno.EHOSTDOWN,
+                                errno.ETIMEDOUT,
+                                errno.ECONNREFUSED,
+                                ssl.SSLEOFError):
+                emsg = ("socket.error = {0}: OutgoerTLS at {1} receiving"
+                        " from {2}\n".format(ex, self.ca, self.ha))
+                console.profuse(emsg)
+                self.cutoff = True  # this signals need to close/reopen connection
+                return bytes()  # data empty
             else:
-                emsg = ("SSLError = {0}: OutgoerTLS at {1} receiving"
+                emsg = ("socket.error = {0}: OutgoerTLS at {1} receiving"
                         " from {2}\n".format(ex, self.ca, self.ha))
                 console.profuse(emsg)
                 raise  # re-raise
 
         if data:  # connection open
             if console._verbosity >= console.Wordage.profuse:  # faster to check
-                cmsg = ("\nOutgoer at {0}, received from {1}:\n------------\n"
-                           "{2}\n".format(self.ca, self.ha, data.decode("UTF-8")))
+                try:
+                    load = data.decode("UTF-8")
+                except UnicodeDecodeError as ex:
+                    load = "0x{0}".format(hexlify(data).decode("ASCII"))
+                cmsg = ("Outgoer at {0}, received from {1}:\n------------\n"
+                        "{2}\n\n".format(self.ca, self.ha, load))
                 console.profuse(cmsg)
 
             if self.wlog:  # log over the wire rx
@@ -620,19 +693,33 @@ class ClientTls(Client):
 
         return data
 
-    def send(self, data, da=None):
+    def send(self, data):
         """
         Perform non blocking send on connected socket .cs.
         Return number of bytes sent
-
         data is string in python2 and bytes in python3
-        da is destination address not used for compatibility interface
         """
         try:
             result = self.cs.send(data) #result is number of bytes sent
-        except ssl.SSLError as ex:
-            result = 0
-            if ex.errno not in (ssl.SSL_ERROR_WANT_READ, ssl.SSL_ERROR_WANT_WRITE):
+        except socket.error as ex:  # ssl.SSLError is a subtype of socket.error
+            # ex.args[0] is always ex.errno for better compat
+            if ex.args[0] in (ssl.SSL_ERROR_WANT_READ, ssl.SSL_ERROR_WANT_WRITE):
+                result = 0
+            elif ex.args[0] in (errno.ECONNRESET,
+                                errno.ENETRESET,
+                                errno.ENETUNREACH,
+                                errno.EHOSTUNREACH,
+                                errno.ENETDOWN,
+                                errno.EHOSTDOWN,
+                                errno.ETIMEDOUT,
+                                errno.ECONNREFUSED,
+                                ssl.SSLEOFError):
+                emsg = ("socket.error = {0}: OutgoerTLS at {1} while sending "
+                        "to {2} \n".format(ex, self.ca, self.ha))
+                console.profuse(emsg)
+                self.cutoff = True  # this signals need to close/reopen connection
+                result = 0
+            else:
                 emsg = ("socket.error = {0}: OutgoerTLS at {1} while sending "
                         "to {2} \n".format(ex, self.ca, self.ha))
                 console.profuse(emsg)
@@ -640,8 +727,12 @@ class ClientTls(Client):
 
         if result:
             if console._verbosity >=  console.Wordage.profuse:
-                cmsg = ("\nOutgoer at {0}, sent {1} bytes to {2}:\n------------\n"
-                        "{3}\n".format(self.ca, result, self.ha, data[:result].decode('UTF-8')))
+                try:
+                    load = data[:result].decode("UTF-8")
+                except UnicodeDecodeError as ex:
+                    load = "0x{0}".format(hexlify(data[:result]).decode("ASCII"))
+                cmsg = ("Outgoer at {0}, sent {1} bytes to {2}:\n------------\n"
+                        "{3}\n\n".format(self.ca, result, self.ha, load))
                 console.profuse(cmsg)
 
             if self.wlog:
